@@ -98,8 +98,6 @@ class CompletionService:
                                       verbose=self.verbose_level)
 
         self.queue_name = self.queue.get_name()
-        if self.clean_queue:
-            self.queue.clean()
 
         self.submitted = {}
         self.use_spot = cloud_name in ['ec2spot', 'gcspot']
@@ -114,7 +112,7 @@ class CompletionService:
             pass
         self.p = None
         if self.wm:
-            self.logger.debug('Spinning up cloud workers')
+            self.logger.info('Spinning up cloud workers')
             if self.use_spot:
                 self.wm.start_spot_workers(
                     self.queue_name,
@@ -133,12 +131,13 @@ class CompletionService:
                         timeout=self.cloud_timeout)
 
         elif self.queue_name is None or self.queue_name == 'local':
-            self.logger.debug('Starting local worker')
+            self.logger.info('Starting local worker')
             self.p = subprocess.Popen([
                 'studio-local-worker',
                 '--verbose=%s' % self.config['verbose'],
                 '--timeout=' + str(self.cloud_timeout)],
                 close_fds=True)
+            self.logger.info('Local worker started')
 
         # yet another case is when queue name is specified, but
         # cloud is not - that means running on a separately
@@ -154,9 +153,8 @@ class CompletionService:
 
     def close(self, delete_queue=True):
         self.logger.info("Studioml completion service shutting down")
-        # if self.queue_name != 'local' and delete_queue:
         request_delete_queue = self.shutdown_del_queue or delete_queue
-        self.queue.shutdown(request_delete_queue)
+        runner.shutdown_queue(self.queue, self.logger, request_delete_queue)
 
     def submitTaskWithFiles(
             self,
@@ -249,11 +247,13 @@ class CompletionService:
                                     data = pickle.load(f)
 
                                 del self.submitted[e.key]
+                                self.logger.info("Got back result for experiment {0}"
+                                                 .format(e.key))
                                 return (e.key, data)
                     except BaseException as e:
-                        self.logger.debug(
+                        self.logger.info(
                             "Getting result failed due to exception:")
-                        self.logger.debug(e)
+                        self.logger.info(e)
 
                     '''
                     if e is not None and e.status == 'finished':
@@ -274,6 +274,7 @@ class CompletionService:
 
             if timeout == 0 or \
                (timeout > 0 and total_sleep_time > timeout):
+                self.logger.info("Timeout not set or exceeded. Returning with no result.")
                 return None
 
             if self.p is not None:
