@@ -10,19 +10,19 @@ import sys
 import shutil
 import subprocess
 import os
+import threading
 import numpy as np
 import requests
 import six
-
-from tensorflow.core.util import event_pb2
-
-import boto3
+from botocore.exceptions import ClientError
+from pygtail import Pygtail
+from . import model_setup
+from .base_artifact_store import BaseArtifactStore
 
 DAY = 86400
 HOUR = 3600
 MINUTE = 60
 
-
 def remove_backspaces(line):
     splitline = re.split('(\x08+)', line)
     try:
@@ -41,11 +41,14 @@ def remove_backspaces(line):
 
 
 def sha256_checksum(filename, block_size=65536):
-    sha256 = hashlib.sha256()
+    return filehash(filename, block_size, hashobj=hashlib.sha256())
+
+
+def filehash(filename, block_size=65536, hashobj=hashlib.sha256()):
     with open(filename, 'rb') as f:
         for block in iter(lambda: f.read(block_size), b''):
-            sha256.update(block)
-    return sha256.hexdigest()
+            hashobj.update(block)
+    return hashobj.hexdigest()
 
 
 def rand_string(length):
@@ -54,6 +57,7 @@ def rand_string(length):
 
 
 def event_reader(fileobj):
+    from tensorflow.core.util import event_pb2
 
     if isinstance(fileobj, str):
         fileobj = open(fileobj, 'rb')
@@ -82,18 +86,30 @@ def event_reader(fileobj):
 
 
 def rsync_cp(source, dest, ignore_arg='', logger=None):
-    if os.path.exists(dest):
-        shutil.rmtree(dest) if os.path.isdir(dest) else os.remove(dest)
-    os.makedirs(dest)
+    try:
+        if os.path.exists(dest):
+            shutil.rmtree(dest) if os.path.isdir(dest) else os.remove(dest)
+        os.makedirs(dest)
+    except OSError:
+        pass
 
     if ignore_arg != '':
         source += "/"
         tool = 'rsync'
         args = [tool, ignore_arg, '-aHAXE', source, dest]
     else:
-        os.rmdir(dest)
+        try:
+            os.rmdir(dest)
+        except OSError:
+            pass
+
         tool = 'cp'
-        args = [tool, '-pR', source, dest]
+        args = [
+            tool,
+            '-pR',
+            source,
+            dest
+        ]
 
     pcp = subprocess.Popen(args, stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
@@ -223,6 +239,9 @@ class Progbar(object):
 
 
 def download_file(url, local_path, logger=None):
+    if url.startswith('s3://') or url.startswith('gs://'):
+        return download_file_from_qualified(url, local_path, logger)
+
     response = requests.get(
         url,
         stream=True)
@@ -256,6 +275,20 @@ def upload_file(url, local_path, logger=None):
         logger.debug('File upload done in {} s'
                      .format(time.time() - tic))
 
+def _looks_like_url(name):
+    """
+    Function tries to determine if input argument
+    looks like URL and not like S3 bucket name.
+    :param name - input name
+    :return: True, if name looks like URL;
+             False otherwise.
+    """
+    if name.endswith('.com'):
+        return True
+    if name.find(':') >= 0:
+        # Assume it is port number
+        return True
+    return False
 
 def download_file_from_qualified(qualified, local_path, logger=None):
     if qualified.startswith('dockerhub://') or \
@@ -265,8 +298,13 @@ def download_file_from_qualified(qualified, local_path, logger=None):
     assert qualified.startswith('s3://') or \
         qualified.startswith('gs://')
 
-    bucket = qualified.split('/')[2]
-    key = '/'.join(qualified.split('/')[3:])
+    qualified_split = qualified.split('/')
+    if _looks_like_url(qualified_split[2]):
+        bucket = qualified_split[3]
+        key = '/'.join(qualified_split[4:])
+    else:
+        bucket = qualified_split[2]
+        key = '/'.join(qualified_split[3:])
 
     if logger is not None:
         logger.debug(('Downloading file from bucket {} ' +
@@ -274,13 +312,83 @@ def download_file_from_qualified(qualified, local_path, logger=None):
                      .format(bucket, key, local_path))
 
     if qualified.startswith('s3://'):
-        boto3.client('s3').download_file(bucket, key, local_path)
+
+        if qualified.endswith('/'):
+            # subprocess.Popen(
+            #    [
+            #        'aws', 's3', 'cp', '--recursive',
+            #        "s3://{}/{}".format(bucket, key),
+            #        local_path
+            #    ]
+            # ).communicate()
+            _s3_download_dir(bucket, key, local_path, logger=logger)
+        else:
+            s3_client = _get_active_s3_client()
+            s3_client.download_file(bucket, key, local_path)
     else:
         raise NotImplementedError
 
+def _get_active_s3_client():
+    artifact_store = model_setup.get_artifact_store()
+    if artifact_store is None \
+        or not isinstance(artifact_store, BaseArtifactStore):
+        raise NotImplementedError("Artifact store is not set up or has the wrong type")
+
+    storage_client = ((BaseArtifactStore)(artifact_store)).get_storage_client()
+    if storage_client is None:
+        raise NotImplementedError("Expected boto3 storage client for current artifact store")
+    return storage_client
+
+def _s3_download_dir(bucket, dist, local, logger=None):
+    s3_client = _get_active_s3_client()
+
+    paginator = s3_client.get_paginator('list_objects')
+    for result in paginator.paginate(
+            Bucket=bucket,
+            Delimiter='/',
+            Prefix=dist):
+        if result.get('CommonPrefixes') is not None:
+            for subdir in result.get('CommonPrefixes'):
+                _s3_download_dir(
+                    bucket,
+                    subdir.get('Prefix'),
+                    os.path.join(local, subdir.get('Prefix'))
+                )
+
+        if result.get('Contents') is not None:
+            for file in result.get('Contents'):
+                if not os.path.exists(
+                    os.path.dirname(
+                        local +
+                        os.sep +
+                        file.get('Key'))):
+                    os.makedirs(
+                        os.path.dirname(
+                            local +
+                            os.sep +
+                            file.get('Key')))
+
+                try:
+                    key = file.get('Key')
+                    local_path = os.path.join(
+                        local,
+                        re.sub('^' + dist, '', file.get('Key'))
+                    )
+
+                    if logger:
+                        logger.debug(
+                            'Downloading {}/{} to {}'
+                            .format(bucket, key, local_path))
+
+                    s3_client.download_file(bucket, key, local_path)
+                except ClientError as e:
+                    if logger:
+                        logger.debug(
+                            'Download failed with exception {}'.format(e))
+
 
 def has_aws_credentials():
-    return boto3.client('s3')._request_signer._credentials is not None
+    return _get_active_s3_client()._request_signer._credentials is not None
 
 
 def retry(f,
@@ -290,6 +398,9 @@ def retry(f,
         try:
             return f()
         except exception_class as e:
+            if i == no_retries - 1:
+                raise e
+
             if logger:
                 logger.info(
                     ('Exception {} is caught, ' +
@@ -365,6 +476,15 @@ def sixdecode(s):
     raise TypeError("Unknown type of " + str(s))
 
 
+def shquote(s):
+    try:
+        import pipes as P
+    except ImportError:
+        import shlex as P
+
+    return P.quote(s)
+
+
 duration_regex = re.compile(
     r'((?P<hours>-?\d+?)h)?((?P<minutes>-?\d+?)m)?((?P<seconds>-?\d+?)s)?')
 
@@ -388,3 +508,46 @@ def parse_duration(duration_str):
 
 def str2duration(s):
     return parse_duration(s.lower())
+
+
+def rm_rf(path):
+    '''
+    remove file or a directory
+    '''
+    if not os.path.exists(path):
+        return
+
+    if os.path.isfile(path):
+        os.remove(path)  # remove the file
+    elif os.path.isdir(path):
+        shutil.rmtree(path)  # remove dir and all contains
+    else:
+        raise ValueError("file {} is not a file or dir.".format(path))
+
+class LogReprinter(object):
+    def __init__(self, log_path):
+        self.logpath = log_path
+        self.logtail = Pygtail(log_path)
+        self.is_running = False
+        self.tail_thread = None
+
+    def run(self):
+        self.tail_thread =\
+            threading.Thread(target=self.tail_func, daemon=True)
+        self.is_running = True
+        self.tail_thread.start()
+
+    def stop(self):
+        self.is_running = False
+        if self.tail_thread is not None:
+            self.tail_thread.join()
+
+    def tail_func(self):
+        while self.is_running and self.logtail:
+            for line in self.logtail:
+                if self.is_running:
+                    print(line)
+                else:
+                    return
+            time.sleep(0.1)
+
